package edu.ntnu.stud.chaosgame.view;

import edu.ntnu.stud.chaosgame.controller.PopupManager;
import javafx.scene.image.ImageView;
import javafx.scene.input.ScrollEvent;
import javafx.scene.transform.Affine;

/**
 * This class extends ImageView to implement proper zooming and panning
 * according to the requirements of the chaos game.
 */
public class ChaosGameImageView extends ImageView {

  private final ChaosGameGuiView controller;
  private final Affine transform = new Affine();

  private double lastCentreX;
  private double lastCentreY;
  private double startX;
  private double startY;

  public ChaosGameImageView(ChaosGameGuiView controller) {
    this.setOnScroll(this::zoom);
    this.setOnMousePressed(this::mousePressed);
    this.setOnMouseDragged(this::mouseDragged);
    this.setOnMouseReleased(this::mouseReleased);
    this.getTransforms().add(transform);
    this.controller = controller;
    this.lastCentreX = (float) controller.getWidth() / 2;
    this.lastCentreY = (float) -controller.getHeight() / 2;

    //this.setStyle("-fx-background-color: white;");
  }

  /**
   * Zooms the image view in or out based on the scroll event.
   *
   * @param event the event.
   */
  private synchronized void zoom(ScrollEvent event) {
    double zoomFactor = event.getDeltaY() > 0 ? 1.20 : 1 / 1.05;
    try {
      // Get the old values
      double oldScaleX = transform.getMxx();
      double oldScaleY = transform.getMyy();
      double oldTranslateX = transform.getTx();
      double oldTranslateY = transform.getTy();

      // Compute the new values
      double newScaleX = oldScaleX * zoomFactor;
      double newScaleY = oldScaleY * zoomFactor;
      double newTranslateX = oldTranslateX - (event.getX() * (newScaleX - oldScaleX));
      double newTranslateY = oldTranslateY - (event.getY() * (newScaleY - oldScaleY));

      // Update the transform
      transform.setMxx(newScaleX);
      transform.setMyy(newScaleY);
      transform.setTx(newTranslateX);
      transform.setTy(newTranslateY);
    } catch (Exception e) {
      PopupManager.displayError("Zoom error", e.getMessage());
    }
  }

  private synchronized void mousePressed(javafx.scene.input.MouseEvent event) {
    startX = event.getX();
    startY = event.getY();
  }

  private synchronized void mouseDragged(javafx.scene.input.MouseEvent event) {
    double deltaX = event.getX() - startX;
    double deltaY = event.getY() - startY;

    transform.setTx(transform.getTx() + deltaX);
    transform.setTy(transform.getTy() + deltaY);
  }

  private synchronized void mouseReleased(javafx.scene.input.MouseEvent event) {
    double deltaX = event.getX() - startX;
    double deltaY = event.getY() - startY;


    lastCentreX += deltaX;
    lastCentreY += deltaY;
  }

}
